---
title: "Logs on S3"
meta_title: "AWS S3"
meta_description: "Using AWS S3 buckets to organize your jobs, builds, and experiment logs. Polyaxon allows users to manage all logs generated by jobs, builds, and experiments containers in S3."
custom_excerpt: "Amazon S3 has a simple web services interface that you can use to store and retrieve any amount of data, at any time, from anywhere on the web. It gives any developer access to the same highly scalable, reliable, fast, inexpensive data storage infrastructure that Amazon uses to run its own global network of web sites."
image: "../../content/images/integrations/s3.png"
author:
  name: "Polyaxon"
  slug: "Polyaxon"
  website: "https://polyaxon.com"
  twitter: "polyaxonAI"
  github: "polyaxon"
tags: 
  - logging
  - storage
featured: false
visibility: public
status: published
---

Polyaxon allows users to manage all logs generated by jobs, builds, and experiments containers in Amazon S3.

## Create an S3 bucket

You should create an S3 bucket (e.g. plx-storage). 

In order to use S3 buckets with Polyaxon, you should create a file containing your access information json object, e.g. `s3-key.json`.
This file should include at least the following information:

```json
{
  "AWS_ACCESS_KEY_ID" : "",
  "AWS_SECRET_ACCESS_KEY": ""
}
```

All possible values:

```json
{
  "AWS_ENDPOINT_URL": "",
  "AWS_ACCESS_KEY_ID": "",
  "AWS_SECRET_ACCESS_KEY": "",
  "AWS_SECURITY_TOKEN": "",
  "AWS_REGION": ""
}
```

## Create a secret on Kubernetes

You should then create a secret with this access keys information on Kubernetes on the same namespace as Polyaxon deployment:

`kubectl create secret generic s3-secret --from-file=s3-secret.json=path/to/s3-key.json -n polyaxon`

## Use the secret name and secret key in your logs persistence definition

```yaml
persistence:
  logs:
    store: s3
    bucket: s3://[BUCKET-NAME]
    secret: [SECRET-NAME]
    secretKey: [SECRET-KEY]
```

e.g.

```yaml
persistence:
  logs:
    store: s3
    bucket: s3://logs-bucket
    secret: s3-secret
    secretKey: s3-key
```

## Update/Install Polyaxon deployment

You can now [install](/setup/kubernetes/)/[upgrade](/setup/kubernetes/#upgrade-polyaxon) Polyaxon with access the outputs on S3.
